char * copyStringUsingArrayNotation(const char * src) {
	int i = 0;
	while (src[i] != '\0') {
		++i;
	}
	char * dest = new char[i + 1];
	i = 0;
	while (src[i] != '\0') {
		dest[i] = src[i];
		++i;
	}
	dest[i] = '\0';
	return dest;
}

// Pointers can be interpreted as arrays - in which case the pointer points to the first element of the array.
// Pointer arithmetic: By adding or subtracting an integer to/from a pointer, 
// you get a new pointer that has been shifted to the right or to the left of the original one. For instance, 
// if 'src' points to the first (index 0) element of an array, then src + 5 will point to the sixth (index 5) element.
// ++ and -- may be used to move a pointer one element to the right or left.
// The * operator will dereference a pointer - that is, return the element that the pointer points to.
// Subtracting two pointers will tell you the "distance" between the pointers - how many elements there are from the first one to the second one.
// Writing src[i] is the same as writing *(src + i) - that is, move 'i' steps to the right of 'src' and look at the element there.
char * copyStringUsingPointers(const char * src) {
	const char * end = src;             // Begin by making a new pointer to the first element. const means that we cannot change the data that the pointer points to, but we can change the pointer itself (by making it point to something else).
	while (*end != '\0')                // As long as 'end' does not point to a null terminator...
		++end;                          // Move one step to the right
	int length = end - src;             // 'end' now points at the null terminator. The length of the string is then the difference of the pointers.
	char * dest = new char[length + 1]; // Make a new array for the copy of the string. We need one additional element for the null terminator.
	while (*src != '\0') {              // While 'src' does not point to a null terminator...
		*dest = *src;                   // Copy the char that 'src' points to to the place that 'dest' points to
		src++;                          // Move both pointers one step to the right
		dest++;
	}
	*dest = '\0';                       // Insert the null terminator
	return dest - length;               // 'dest' now points to the null terminator, so we must subtract 'length' from it in order to get a pointer to the start of the string, which we return
}
