----------------------------------------------------------------------------------
-- Company: NTNU
-- Engineer: Stig Fjellskaalnes
-- 
-- Create Date:    13:14:48 09/26/2008 
-- Design Name: 
-- Module Name:    decryption - rtl 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity decryption is
	port (
		clk		:	in std_logic;
		reset		:	in std_logic;
		en_in		:	in std_logic;
		inputs	:	in std_logic_vector (159 downto 0);
		outputs	:	out std_logic_vector (159 downto 0);
		en_out	:	out std_logic
		);
		
end decryption;

architecture rtl of decryption is

	type states is (IDLE,SEND_TO_PM_T,RECV_FROM_PM_T,SEND_TO_DOB,
						RECV_FROM_DOB,SEND_TO_SEQ,RECV_FROM_SEQ,
						SEND_TO_PM_S,RECV_FROM_PM_S,SEND);

	component private_matrix_s
		port (
			clk			:	in std_logic;
			reset			:	in std_logic;
			en_in			:	in std_logic;
			input			:	in std_logic_vector (4 downto 0);
			cnt			:	in std_logic_vector (4 downto 0);
			output		:	out std_logic_vector (159 downto 0);
			en_out		:	out std_logic
		);
	end component;

	component dobbertin_rom			-- defining the Dobbertin component 
											-- in the decryption top level
		port (
			z						:	in std_logic_vector (12 downto 0);
			clk,reset,en_in	:	in std_logic;
			db						: out std_logic_vector(12 downto 0);
			en_out				: out std_logic
		);
	end component;
	
	component sequencer
		port (
				clk 			:	in  std_logic;
				reset 		:	in  std_logic;
				en_in 		:	in  std_logic;
				input_seq	:	in	 std_logic_vector (159 downto 0);
				en_out		:	out std_logic;
				output_seq	:	out std_logic_vector (159 downto 0)	
		);
	end component;
	
	signal state,next_state	:	states;
	--signal dec_input			:	std_logic_vector (159 downto 0);
	signal dec_output			:	std_logic_vector (159 downto 0);
	signal result_pmt			:	std_logic_vector (159 downto 0);
	signal result_pms			:	std_logic_vector (159 downto 0);
	signal result_seq			:	std_logic_vector (159 downto 0);
	signal seq_in				:	std_logic_vector (159 downto 0);
	signal dob_vector_in		:	std_logic_vector (12 downto 0);
	signal dob_vector_out	:	std_logic_vector (12 downto 0);
	signal count_t				:	std_logic_vector (4 downto 0);					-- counter private_matrix t
	signal count_s				:	std_logic_vector (4 downto 0);					-- counter private_matrix s
	signal shift_pmt			:	std_logic_vector (4 downto 0);
	signal shift_pms			:	std_logic_vector (4 downto 0);
	signal en_in_pm_t			:	std_logic;
	signal en_in_pm_s			:	std_logic;
	signal en_in_dob			:	std_logic;
	signal en_in_seq			:	std_logic;
	signal en_out_seq			:	std_logic;
	signal en_out_pm_t		:	std_logic;
	signal en_out_pm_s		:	std_logic;
	signal en_out_dob			:	std_logic;
	
begin
	DR:	DOBBERTIN_ROM				-- initializing the Dobbertin 
											-- ROM component in the decryption
											--	circuit
	port map(
		clk			=>	clk,
		reset			=>	reset,
		en_in			=>	en_in_dob,
		en_out		=>	en_out_dob,
		z				=>	dob_vector_in,
		db				=> dob_vector_out
	);
	
	PM_T:	PRIVATE_MATRIX_S			-- initializing the Private Matrix 
											-- (T) component in the decryption
											--	circuit
	port map(
		clk			=>	clk,
		reset			=>	reset,
		en_in			=>	en_in_pm_t,
		input			=>	shift_pmt,
		cnt			=> count_t,
		output		=>	result_pmt,
		en_out		=>	en_out_pm_t
	);
		
	SEQ: SEQUENCER						-- initilizing the sequencer
											-- component in the decryption
											-- circuit
	
	port map(
		clk			=> clk,
		reset			=>	reset,
		en_in			=> en_in_seq,
		input_seq	=> seq_in,
		en_out		=> en_out_seq,
		output_seq	=> result_seq
		);

	PM_S:	PRIVATE_MATRIX_S			-- initializing the Private Matrix 
											-- (T) component in the decryption
											--	circuit
	port map(
		clk			=>	clk,
		reset			=>	reset,
		en_in			=>	en_in_pm_s,
		input			=>	shift_pms,
		cnt			=> count_s,
		output		=>	result_pms,
		en_out		=>	en_out_pm_s
		);

	running: process (clk, en_in)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				state <= IDLE;
				
			elsif (en_in = '1') then
				state <= next_state;
				
			end if;
		end if;
	end process;
	
	output_dec: process (clk,en_in,state)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				dob_vector_in <= (others => '0');
				--dec_input <= (others => '0');
				dec_output <= (others => '0');
--				result_pmt <= (others => '0');
--				result_pms <= (others => '0');
--				result_seq <= (others => '0');
				outputs <= (others => '0');
				shift_pmt <= (others => '0');
				shift_pms <= (others => '0');
				en_out <= '0';
				en_in_dob <= '0';
				en_in_pm_t <= '0';
				en_in_pm_s <= '0';
--				en_out_dob <= '0';
--				en_out_seq <= '0';
				en_in_seq <= '0';
				count_t <= (others => '0');
				count_s <= (others => '0');

				seq_in <= (others => '0');
				
		
			elsif (en_in = '1') then
				case (state) is
					when IDLE =>
					
						--dec_input <= "1001101101001111001100101100110110011011010011110011001011001101100110110100111100110010110011011001101101001111001100101100110110011011010011110011001011001101";
						en_in_pm_t <= '0';
						if (count_t <= "11111") then
							count_t <= (others => '0');
						end if;
					
					
					when SEND_TO_PM_T =>
						en_in_pm_t <= '1';
						if (count_t = "11111" and en_out_pm_t = '1') then
							null;
						else
							shift_pmt <= inputs ((4+(conv_integer(count_t)*5)) downto (0+(conv_integer(count_t)*5)));
							if (en_out_pm_t = '1') then
								count_t <= count_t + 1;
							end if;
						end if;
					
					when RECV_FROM_PM_T =>
						
						case (result_pmt) is
							when ("ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ") =>
								null;
							when others =>
								en_in_dob <= '1';		
						end case;
						
					when SEND_TO_DOB =>
						en_in_pm_t <= '0';
						en_in_dob <= '1';
						dob_vector_in(5 downto 0) <= result_pmt(5 downto 0);
						dob_vector_in(6) <= result_pmt(10);
						dob_vector_in(7) <= result_pmt(15);
						dob_vector_in(8) <= result_pmt(20);
						dob_vector_in(9) <= result_pmt(25);
						dob_vector_in(10) <= result_pmt(30);
						dob_vector_in(11) <= result_pmt(35);
						dob_vector_in(12) <= result_pmt(40);
						
					when RECV_FROM_DOB =>
						if (en_out_dob = '1') then
							seq_in <= result_pmt;
							case (dob_vector_out) is
								when "ZZZZZZZZZZZZZ" =>
									null;
																		
								when others =>
									seq_in(5 downto 0) <= dob_vector_out(5 downto 0);
									seq_in(10) <= dob_vector_out(6);
									seq_in(15) <= dob_vector_out(7);
									seq_in(20) <= dob_vector_out(8);
									seq_in(25) <= dob_vector_out(9);
									seq_in(30) <= dob_vector_out(10);
									seq_in(35) <= dob_vector_out(11);
									seq_in(40) <= dob_vector_out(12);
									--en_out_dob <= '0';
							end case;
						end if;
						
					when SEND_TO_SEQ =>
						en_in_seq <= '1';
						en_in_dob <= '0';
					when RECV_FROM_SEQ =>
						en_in_seq <= '0';
					when SEND_TO_PM_S =>
						en_in_pm_s <= '1';
						if (count_s = "11111" and en_out_pm_s = '1') then
							null;
						else
							shift_pms <= result_seq(((4+(conv_integer(count_s)*5))) downto (0+((conv_integer(count_s)*5))));
							if (en_out_pm_s = '1') then
								count_s <= count_s + 1;
							end if;
						end if;
						
					when RECV_FROM_PM_S =>
						en_in_pm_s <= '0';
						dec_output <= result_pms;
						
					when SEND =>
						outputs <= dec_output;
					when others =>
						null;
				end case;
			end if;
		end if;
	end process;
	
	-- calculates next state
	
	next_state_dec: process(state,count_t,en_out_pm_t,
							en_out_seq,count_s,en_out_pm_s)
	begin
		next_state <= state;
		
		case (state) is
			-- 
			--
			when IDLE =>
				next_state <= SEND_TO_PM_T;

			-- receives data from private matrix only when
			-- counter reaches 31, when the private matrix is done
			-- processing the 160bit data
			when SEND_TO_PM_T =>			
				if (count_t = 31 and en_out_pm_t = '1') then
						next_state <= RECV_FROM_PM_T;
					else
						next_state <= SEND_TO_PM_T;

				end if;
			
			-- if the data is not valid, new data is being imported
			-- and the state machine moves back to IDLE.
			-- else, send data to dobbertin ROM
			when RECV_FROM_PM_T =>
				case result_pmt is
					when ("ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ") =>
						next_state <= IDLE;
					when others =>
						next_state <= SEND_TO_DOB;
				end case;
				
			when SEND_TO_DOB =>
					next_state <= RECV_FROM_DOB;
				
			when RECV_FROM_DOB =>
				next_state <= SEND_TO_SEQ;
				
			when SEND_TO_SEQ =>
				case (en_out_seq) is
					when '0' =>
						next_state <= SEND_TO_SEQ;
					when '1' =>
						next_state <= RECV_FROM_SEQ;
					when others =>
						next_state <= IDLE;
				end case;
				
			when RECV_FROM_SEQ =>
				next_state <= SEND_TO_PM_S;
			when SEND_TO_PM_S =>
				if (count_s = 31 and en_out_pm_s = '1') then
					next_state <= RECV_FROM_PM_S;
				else
					next_state <= SEND_TO_PM_S;
				end if;
				
			when RECV_FROM_PM_S =>
				case result_pmt is
					when ("ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ") =>
						next_state <= IDLE;
					when others =>
						next_state <= SEND;
				end case;
				
			when SEND =>
				next_state <= IDLE;
				
			when others =>
				next_state <= IDLE;
				
		end case;
	end process;
	
	
end rtl;

