library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity encryption is

	port ( 
		clk		:	in std_logic;
		reset		:	in std_logic;
		en_in		:	in std_logic;
		datain	:	in std_logic_vector(159 downto 0);
		dataout	:	out std_logic_vector(159 downto 0)
	);
end encryption;

architecture rtl of encryption is
	type states is (IDLE,EXPAND,PUBL_MATR,RES);

	component expander 
		port (
			clk,reset,en_in	:	in std_logic;
			input					:	in std_logic_vector(159 downto 0);
			output_1				:	out std_logic_vector(159 downto 0);
			output_2				:	out std_logic_vector(80 downto 0);
			en_out				:	out std_logic
	
		);
	end component;
	
	component public_matrix 
		port (
			clk,reset,en_in	:	in std_logic;
			input_1				:	in std_logic_vector(159 downto 0);
			input_2				:	in std_logic_vector(80 downto 0);
			cnt					:	in integer range 0 to 80;
			output				:	out std_logic_vector(159 downto 0);
			en_out				:	out std_logic;
			done					:	out std_logic
		);
	end component;
	
	signal state,next_state		:	states;
	signal cnt							:	integer range 0 to 80;
	signal en_in_exp,en_in_pub	:	std_logic;
	signal en_out_exp,en_out_pub	:	std_logic;
	signal first_data					:	std_logic_vector(159 downto 0);
	signal second_data				:	std_logic_vector(80 downto 0);
	signal final_result				:	std_logic_vector(159 downto 0);
	signal ready						:	std_logic;
	
begin

	EXP:		expander
	port map(
		clk		=> clk,
		reset		=> reset,
		en_in		=> en_in_exp,
		input		=> datain,
		output_1	=> first_data,
		output_2	=>	second_data,
		en_out	=>	en_out_exp
	);

	PUB_MATR:	public_matrix
	port map (
		clk		=> clk,
		reset		=> reset,
		en_in		=>	en_in_pub,
		input_1	=> first_data,
		input_2	=> second_data,
		cnt		=> cnt,
		output	=>	final_result,
		en_out	=> en_out_pub,
		done		=> ready
	);
	sync_run:	process(clk)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				state <= IDLE;
				dataout <= (others => '0');
				
			elsif (en_in = '1') then
				state <= next_state;
				dataout <= final_result;
			end if;
		end if;
	end process;

	output_decode:	process(clk,state)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				en_in_exp <= '0';
				en_in_pub <= '0';
				
			elsif (en_in = '1') then
				case (state) is
					when IDLE			=>
						en_in_exp <= '0';
						en_in_pub <= '0';
						
					when EXPAND			=>
						en_in_exp <= '1';
						en_in_pub <= '0';
						
					when PUBL_MATR		=>
						en_in_exp <= '0';
						en_in_pub <= '1';
						if (en_out_exp = '1') then
							cnt <= cnt + 1;
							if (cnt = 79) then
								cnt <= cnt + 0;
							end if;
						end if;
					when RES				=>
						--dataout <= final_result;
					when others =>
						null;
				end case;
			end if;
		end if;
	end process;
	
	next_state_decode: process(state,en_out_exp,ready)
	begin
		next_state <= state;
		case (state) is
			when IDLE			=>
				next_state <= EXPAND;
				
			when EXPAND			=>
				if (en_out_exp <= '1') then
					next_state <= PUBL_MATR;
				elsif (ready = '1') then
					next_state <= RES;
				else
					next_state <= EXPAND;
				end if;
			when PUBL_MATR		=>
				if (en_out_pub <= '1') then
					next_state <= EXPAND;
				elsif (ready = '1') then
					next_state <= RES;
				else
					next_state <= PUBL_MATR;
				end if;
			when RES				=>
				next_state <= IDLE;
			when others			=>
				null;
		end case;
	end process;
end rtl;

