library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity expander is
	port (
		clk,reset,en_in	:	in std_logic;
		input					:	in std_logic_vector(159 downto 0);
		output_1				:	out std_logic_vector(159 downto 0);
		output_2				:	out std_logic_vector(80 downto 0);
		en_out				:	out std_logic
	);

end expander;

architecture rtl of expander is




	type states is (IDLE,COMB,SEL);
	signal state,next_state	:	states;
	
	signal vector				:	std_logic_vector (12880 downto 0);
	signal tmp					:	std_logic_vector (159 downto 0);
	signal inc					:	integer range 0 to 12880;

begin

	run:	process(clk)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				state <= IDLE;
			elsif (en_in = '1') then
				state <= next_state;
			end if;
		end if;
	end process;

	output_decode:	process (clk,state)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				vector <= (others => '0');
				en_out <= '0';
				tmp <= (others => '0');
				inc <= 0;
				output_1 <= (others => '0');
				output_2 <= (others => '0');
			elsif (en_in = '1') then
				case (state) is
					when IDLE =>
						tmp <= input;
						en_out <= '0';
					when COMB	=>
						vector(0) 							<= '1';
						vector(160 downto 1) 			<= tmp; 														 --160
						vector(319 downto 161)			<= tmp(158 downto 0) and input(159 downto 1); --159
						vector(477 downto 320)			<= tmp(157 downto 0) and input(159 downto 2); --158
						vector(634 downto 478)			<= tmp(156 downto 0) and input(159 downto 3); --157
						vector(790 downto 635)			<= tmp(155 downto 0) and input(159 downto 4); --156
						vector(945 downto 791)			<= tmp(154 downto 0) and input(159 downto 5); --155
						vector(1099 downto 946)		<= tmp(153 downto 0) and input(159 downto 6); --154
						vector(1252 downto 1100)		<= tmp(152 downto 0) and input(159 downto 7); --153
						vector(1404 downto 1253)		<= tmp(151 downto 0) and input(159 downto 8); --152
						vector(1555 downto 1405)		<= tmp(150 downto 0) and input(159 downto 9); --151
						vector(1705 downto 1556)		<= tmp(149 downto 0) and input(159 downto 10);--150
						vector(1854 downto 1706)		<= tmp(148 downto 0) and input(159 downto 11);--149
						vector(2002 downto 1855)		<= tmp(147 downto 0) and input(159 downto 12);--148
						vector(2149 downto 2003)		<= tmp(146 downto 0) and input(159 downto 13);--147
						vector(2295 downto 2150)		<= tmp(145 downto 0) and input(159 downto 14);--146
						vector(2440 downto 2296)		<= tmp(144 downto 0) and input(159 downto 15);--145
						vector(2584 downto 2441)		<= tmp(143 downto 0) and input(159 downto 16);--144
						vector(2727 downto 2585)		<= tmp(142 downto 0) and input(159 downto 17);--143
						vector(2869 downto 2728)		<= tmp(141 downto 0) and input(159 downto 18);--142
						vector(3010 downto 2870)		<= tmp(140 downto 0) and input(159 downto 19);--141
						vector(3150 downto 3011)		<= tmp(139 downto 0) and input(159 downto 20);--140
						vector(3289 downto 3151)		<= tmp(138 downto 0) and input(159 downto 21);--139
						vector(3427 downto 3290)		<= tmp(137 downto 0) and input(159 downto 22);--138
						vector(3564 downto 3428)		<= tmp(136 downto 0) and input(159 downto 23);--137
						vector(3700 downto 3565)		<= tmp(135 downto 0) and input(159 downto 24);--136
						vector(3835 downto 3701)		<= tmp(134 downto 0) and input(159 downto 25);--135
						vector(3969 downto 3836)		<= tmp(133 downto 0) and input(159 downto 26);--134
						vector(4102 downto 3970)		<= tmp(132 downto 0) and input(159 downto 27);--133
						vector(4234 downto 4103)		<= tmp(131 downto 0) and input(159 downto 28);--132
						vector(4365 downto 4235)		<= tmp(130 downto 0) and input(159 downto 29);--131
						vector(4495 downto 4366)		<= tmp(129 downto 0) and input(159 downto 30);--130
						vector(4624 downto 4496)		<= tmp(128 downto 0) and input(159 downto 31);--129
						vector(4752 downto 4625)		<= tmp(127 downto 0) and input(159 downto 32);--128
						vector(4879 downto 4753)		<= tmp(126 downto 0) and input(159 downto 33);--127
						vector(5005 downto 4880)		<= tmp(125 downto 0) and input(159 downto 34);--126
						vector(5130 downto 5006)		<= tmp(124 downto 0) and input(159 downto 35);--125
						vector(5254 downto 5131)		<= tmp(123 downto 0) and input(159 downto 36);--124
						vector(5377 downto 5255)		<= tmp(122 downto 0) and input(159 downto 37);--123
						vector(5499 downto 5378)		<= tmp(121 downto 0) and input(159 downto 38);--122
						vector(5620 downto 5500)		<= tmp(120 downto 0) and input(159 downto 39);--121
						vector(5740 downto 5621)		<= tmp(119 downto 0) and input(159 downto 40);--120
						vector(5859 downto 5741)		<= tmp(118 downto 0) and input(159 downto 41);--119
						vector(5977 downto 5860)		<= tmp(117 downto 0) and input(159 downto 42);--118
						vector(6094 downto 5978)		<= tmp(116 downto 0) and input(159 downto 43);--117
						vector(6210 downto 6095)		<= tmp(115 downto 0) and input(159 downto 44);--116
						vector(6325 downto 6211)		<= tmp(114 downto 0) and input(159 downto 45);--115
						vector(6439 downto 6326)		<= tmp(113 downto 0) and input(159 downto 46);--114
						vector(6552 downto 6440)		<= tmp(112 downto 0) and input(159 downto 47);--113
						vector(6664 downto 6553)		<= tmp(111 downto 0) and input(159 downto 48);--112
						vector(6775 downto 6665)		<= tmp(110 downto 0) and input(159 downto 49);--111
						vector(6885 downto 6776)		<= tmp(109 downto 0) and input(159 downto 50);--110
						vector(6994 downto 6886)		<= tmp(108 downto 0) and input(159 downto 51);--109
						vector(7102 downto 6995)		<= tmp(107 downto 0) and input(159 downto 52);--108
						vector(7209 downto 7103)		<= tmp(106 downto 0) and input(159 downto 53);--107
						vector(7315 downto 7210)		<= tmp(105 downto 0) and input(159 downto 54);--106
						vector(7420 downto 7316)		<= tmp(104 downto 0) and input(159 downto 55);--105
						vector(7524 downto 7421)		<= tmp(103 downto 0) and input(159 downto 56);--104
						vector(7627 downto 7525)		<= tmp(102 downto 0) and input(159 downto 57);--103
						vector(7729 downto 7628)		<= tmp(101 downto 0) and input(159 downto 58);--102
						vector(7830 downto 7730)		<= tmp(100 downto 0) and input(159 downto 59);--101
						vector(7930 downto 7831)		<= tmp(99 downto 0) and input(159 downto 60);-- 100
						vector(8029 downto 7931)		<= tmp(98 downto 0) and input(159 downto 61);--  99
						vector(8127 downto 8030)		<= tmp(97 downto 0) and input(159 downto 62);--  98
						vector(8224 downto 8128)		<= tmp(96 downto 0) and input(159 downto 63);--  97
						vector(8320 downto 8225)		<= tmp(95 downto 0) and input(159 downto 64);--  96
						vector(8415 downto 8321)		<= tmp(94 downto 0) and input(159 downto 65);--  95
						vector(8509 downto 8416)		<= tmp(93 downto 0) and input(159 downto 66);--  94
						vector(8602 downto 8510)		<= tmp(92 downto 0) and input(159 downto 67);--  93
						vector(8694 downto 8603)		<= tmp(91 downto 0) and input(159 downto 68);--  92
						vector(8785 downto 8695)		<= tmp(90 downto 0) and input(159 downto 69);--  91
						vector(8875 downto 8786)		<= tmp(89 downto 0) and input(159 downto 70);--  90
						vector(8964 downto 8876)		<= tmp(88 downto 0) and input(159 downto 71);--  89
						vector(9052 downto 8965)		<= tmp(87 downto 0) and input(159 downto 72);--  88
						vector(9139 downto 9053)		<= tmp(86 downto 0) and input(159 downto 73);--  87
						vector(9225 downto 9140)		<= tmp(85 downto 0) and input(159 downto 74);--  86
						vector(9310 downto 9226)		<= tmp(84 downto 0) and input(159 downto 75);--  85
						vector(9394 downto 9311)		<= tmp(83 downto 0) and input(159 downto 76);--  84
						vector(9477 downto 9395)		<= tmp(82 downto 0) and input(159 downto 77);--  83
						vector(9559 downto 9478)		<= tmp(81 downto 0) and input(159 downto 78);--  82
						vector(9640 downto 9560)		<= tmp(80 downto 0) and input(159 downto 79);--  81
						vector(9720 downto 9641)		<= tmp(79 downto 0) and input(159 downto 80);--  80
						vector(9799 downto 9721)		<= tmp(78 downto 0) and input(159 downto 81);--  79
						vector(9877 downto 9800)		<= tmp(77 downto 0) and input(159 downto 82);--  78
						vector(9954 downto 9878)		<= tmp(76 downto 0) and input(159 downto 83);--  77
						vector(10030 downto 9955)		<= tmp(75 downto 0) and input(159 downto 84);--  76
						vector(10105 downto 10031)	<= tmp(74 downto 0) and input(159 downto 85);--  75
						vector(10179 downto 10106)	<= tmp(73 downto 0) and input(159 downto 86);--  74
						vector(10252 downto 10180)	<= tmp(72 downto 0) and input(159 downto 87);--  73
						vector(10324 downto 10253)	<= tmp(71 downto 0) and input(159 downto 88);--  72
						vector(10395 downto 10325)	<= tmp(70 downto 0) and input(159 downto 89);--  71
						vector(10465 downto 10396)	<= tmp(69 downto 0) and input(159 downto 90);--  70
						vector(10534 downto 10466)	<= tmp(68 downto 0) and input(159 downto 91);--  69
						vector(10602 downto 10535)	<= tmp(67 downto 0) and input(159 downto 92);--  68
						vector(10669 downto 10603)	<= tmp(66 downto 0) and input(159 downto 93);--  67
						vector(10735 downto 10670)	<= tmp(65 downto 0) and input(159 downto 94);--  66
						vector(10800 downto 10736)	<= tmp(64 downto 0) and input(159 downto 95);--  65
						vector(10864 downto 10801)	<= tmp(63 downto 0) and input(159 downto 96);--  64
						vector(10927 downto 10865)	<= tmp(62 downto 0) and input(159 downto 97);--  63
						vector(10989 downto 10928)	<= tmp(61 downto 0) and input(159 downto 98);--  62
						vector(11050 downto 10990)	<= tmp(60 downto 0) and input(159 downto 99);--  61
						vector(11110 downto 11051)	<= tmp(59 downto 0) and input(159 downto 100);-- 60
						vector(11169 downto 11111)	<= tmp(58 downto 0) and input(159 downto 101);-- 59
						vector(11227 downto 11170)	<= tmp(57 downto 0) and input(159 downto 102);-- 58
						vector(11284 downto 11228)	<= tmp(56 downto 0) and input(159 downto 103);-- 57
						vector(11340 downto 11285)	<= tmp(55 downto 0) and input(159 downto 104);-- 56
						vector(11395 downto 11341)	<= tmp(54 downto 0) and input(159 downto 105);-- 55
						vector(11449 downto 11396)	<= tmp(53 downto 0) and input(159 downto 106);-- 54
						vector(11502 downto 11450)	<= tmp(52 downto 0) and input(159 downto 107);-- 53
						vector(11554 downto 11503)	<= tmp(51 downto 0) and input(159 downto 108);-- 52
						vector(11605 downto 11555)	<= tmp(50 downto 0) and input(159 downto 109);-- 51
						vector(11655 downto 11606)	<= tmp(49 downto 0) and input(159 downto 110);-- 50
						vector(11704 downto 11656)	<= tmp(48 downto 0) and input(159 downto 111);-- 49
						vector(11752 downto 11705)	<= tmp(47 downto 0) and input(159 downto 112);-- 48
						vector(11799 downto 11753)	<= tmp(46 downto 0) and input(159 downto 113);-- 47
						vector(11845 downto 11800)	<= tmp(45 downto 0) and input(159 downto 114);-- 46
						vector(11890 downto 11846)	<= tmp(44 downto 0) and input(159 downto 115);-- 45
						vector(11934 downto 11891)	<= tmp(43 downto 0) and input(159 downto 116);-- 44
						vector(11977 downto 11935)	<= tmp(42 downto 0) and input(159 downto 117);-- 43
						vector(12019 downto 11978)	<= tmp(41 downto 0) and input(159 downto 118);-- 42
						vector(12060 downto 12020)	<= tmp(40 downto 0) and input(159 downto 119);-- 41
						vector(12100 downto 12061)	<= tmp(39 downto 0) and input(159 downto 120);-- 40
						vector(12139 downto 12101)	<= tmp(38 downto 0) and input(159 downto 121);-- 39
						vector(12177 downto 12140)	<= tmp(37 downto 0) and input(159 downto 122);-- 38
						vector(12214 downto 12178)	<= tmp(36 downto 0) and input(159 downto 123);-- 37
						vector(12250 downto 12215)	<= tmp(35 downto 0) and input(159 downto 124);-- 36
						vector(12285 downto 12251)	<= tmp(34 downto 0) and input(159 downto 125);-- 35
						vector(12319 downto 12286)	<= tmp(33 downto 0) and input(159 downto 126);-- 34
						vector(12352 downto 12320)	<= tmp(32 downto 0) and input(159 downto 127);-- 33
						vector(12384 downto 12353)	<= tmp(31 downto 0) and input(159 downto 128);-- 32
						vector(12415 downto 12385)	<= tmp(30 downto 0) and input(159 downto 129);-- 31
						vector(12445 downto 12416)	<= tmp(29 downto 0) and input(159 downto 130);-- 30
						vector(12474 downto 12446)	<= tmp(28 downto 0) and input(159 downto 131);-- 29
						vector(12502 downto 12475)	<= tmp(27 downto 0) and input(159 downto 132);-- 28
						vector(12529 downto 12503)	<= tmp(26 downto 0) and input(159 downto 133);-- 27
						vector(12555 downto 12530)	<= tmp(25 downto 0) and input(159 downto 134);-- 26
						vector(12580 downto 12556)	<= tmp(24 downto 0) and input(159 downto 135);-- 25
						vector(12604 downto 12581)	<= tmp(23 downto 0) and input(159 downto 136);-- 24
						vector(12627 downto 12605)	<= tmp(22 downto 0) and input(159 downto 137);-- 23
						vector(12649 downto 12628)	<= tmp(21 downto 0) and input(159 downto 138);-- 22
						vector(12670 downto 12650)	<= tmp(20 downto 0) and input(159 downto 139);-- 21
						vector(12690 downto 12671)	<= tmp(19 downto 0) and input(159 downto 140);-- 20
						vector(12709 downto 12691)	<= tmp(18 downto 0) and input(159 downto 141);-- 19
						vector(12727 downto 12710)	<= tmp(17 downto 0) and input(159 downto 142);-- 18
						vector(12744 downto 12728)	<= tmp(16 downto 0) and input(159 downto 143);-- 17
						vector(12760 downto 12745)	<= tmp(15 downto 0) and input(159 downto 144);-- 16
						vector(12775 downto 12761)	<= tmp(14 downto 0) and input(159 downto 145);-- 15
						vector(12789 downto 12776)	<= tmp(13 downto 0) and input(159 downto 146);-- 14
						vector(12802 downto 12790)	<= tmp(12 downto 0) and input(159 downto 147);-- 13
						vector(12814 downto 12803)	<= tmp(11 downto 0) and input(159 downto 148);-- 12
						vector(12825 downto 12815)	<= tmp(10 downto 0) and input(159 downto 149);-- 11
						vector(12835 downto 12826)	<= tmp(9 downto 0) and input(159 downto 150); -- 10
						vector(12844 downto 12836)	<= tmp(8 downto 0) and input(159 downto 151); --  9
						vector(12852 downto 12845)	<= tmp(7 downto 0) and input(159 downto 152); --  8
						vector(12859 downto 12853)	<= tmp(6 downto 0) and input(159 downto 153); --  7
						vector(12865 downto 12860)	<= tmp(5 downto 0) and input(159 downto 154); --  6
						vector(12870 downto 12866)	<= tmp(4 downto 0) and input(159 downto 155); --  5
						vector(12874 downto 12871)	<= tmp(3 downto 0) and input(159 downto 156); --  4
						vector(12877 downto 12875)	<= tmp(2 downto 0) and input(159 downto 157); --  3
						vector(12879 downto 12878)	<= tmp(1 downto 0) and input(159 downto 158); --  2
						vector(12880)						<= tmp(0) and input(159);							 --  1
						
						
					when SEL	=>
						-- the output_1 register value is determined by
						-- the value of the counter, which increases by 160
						-- each time this state is active
						output_1 <= vector((159 + inc) downto (0 + inc));
						output_2 <= vector(12880 downto 12800);
						if (inc >= 12640) then
							inc <= inc + 0;
						else
							inc <= inc + 160;
						end if;
						en_out <= '1';
				end case;
			end if;
		end if;
	end process;
	
	next_decode: process(state,inc)
	begin
		next_state <= state;
		case (state) is
			when IDLE	=>
				next_state <= COMB;
			when COMB	=>
				next_state <= SEL; 
			when SEL		=>
				next_state <= IDLE;
				--end if;
		end case;
	end process;
end rtl;

